@@ -1,8 +1,8 @@
-import json
 import os
 import sys
 import time
 from dataclasses import dataclass, field
+from datetime import datetime
 from enum import Enum
 from typing import TYPE_CHECKING, Any, Literal
 
@@ -132,20 +132,23 @@ def _wait_for_item_ready(
     """
     Wait for item to be in available status.
     """
-    print(f"Waiting for {item_name} to be ready...")
+    print(f"Waiting for {item_name} to reach status {status}...")
     start_time = time.time()
+
     while time.time() - start_time < timeout:
         try:
-            item = get_func(*get_func_args, client)
+            item = get_func(*get_func_args, client=client)
             if item.status == status:
-                print(f"{item_name.capitalize()} is ready!")
+                print(f"{item_name.capitalize()} status reached!")
                 return
             print(f"Current {item_name} status: {item.status}")
         except Exception as e:
             print(f"Error checking {item_name} status: {e}")
+
         time.sleep(10)
+
     raise TimeoutError(
-        f"{item_name.capitalize()} did not become ready within {timeout} seconds"
+        f"{item_name.capitalize()} did not reach desired status within {timeout} seconds"
     )
 
 
@@ -159,22 +162,32 @@ def _wait_for_item_deleted(
     """
     Wait for volume to be deleted.
     """
+    import ovh
+
     print(f"Waiting for {item_name} to be deleted...")
     start_time = time.time()
     item = None
     deleted = False
+
+    count = 0
+
     while True:
-        time.sleep(10)
+        time.sleep(10 + count)
+        count += 1
+
         try:
-            item = get_func(*get_func_args, client)
+            item = get_func(*get_func_args, client=client)
             if not item:
                 deleted = True
+            elif item and item.status.upper() == "DELETED":
+                deleted = True
             else:
                 print(f"Current {item_name} status: {item.status} ")
         except ovh.exceptions.ResourceNotFoundError:
+            # Item was deleted if not found
             deleted = True
         except Exception as e:
-            # Check if it's a 404-like error
+            # Item was deleted if 404
             if "404" in str(e) or "not found" in str(e).lower():
                 deleted = True
             elif "Expecting value: line 1 column 1 (char 0)" in str(e):
@@ -185,13 +198,13 @@ def _wait_for_item_deleted(
                 deleted = True
             else:
                 print(f"Error checking {item_name} status: {e}")
+
         if deleted:
             print(f"{item_name.capitalize()} deleted successfully!")
             return
+
         if time.time() - start_time > timeout:
-            # We keep on waiting when the item is marked 'deleted' but still available.
-            if item and item.status != "DELETED":
-                break
+            break
 
     raise TimeoutError(
         f"{item_name.capitalize()} was not deleted within {timeout} seconds"
@@ -202,11 +215,11 @@ class Volume(BaseSchema):
     attached_to: list[str]
     availability_zone: str | None
     bootable: bool
-    creation_date: str
+    creation_date: datetime
     description: str
     id: str
     name: str
-    plan_code: str
+    plan_code: str | None
     region: str
     size: int
     status: str
@@ -214,7 +227,7 @@ class Volume(BaseSchema):
 
 
 class VolumeSnapshot(BaseSchema):
-    creation_date: str
+    creation_date: datetime
     description: str
     id: str
     name: str
@@ -235,6 +248,16 @@ class VolumeType(str, Enum):
     high_speed_luks = "high-speed-luks"
 
 
+class VolumeBackup(BaseSchema):
+    creation_date: datetime
+    id: str
+    name: str
+    region: str
+    size: int
+    status: str
+    volume_id: str
+
+
 def volume_create(
     name: str,
     region: str,
@@ -282,7 +305,6 @@ def volume_create(
         payload["snapshotId"] = snapshot_id
 
     print(f"\nCreating OVHcloud volume '{name}'...")
-    print(f"Request Payload: {json.dumps(payload, indent=2)}\n")
 
     path = f"/cloud/project/{client.project_id}/volume"
     volume_data = client.request("POST", path, payload)
@@ -300,7 +322,7 @@ def volume_create(
 
 def volume_get(
     volume_id: str,
-    region: str,
+    region: str | None = None,
     client: OVHClient | None = None,
 ) -> Volume | None:
     """
@@ -319,7 +341,8 @@ def volume_get(
 
     response = client.request("GET", path)
     volume_data = response.copy()
-    volume_data["region"] = region
+    if region:
+        volume_data["region"] = region
 
     if volume_data.get("status") == "DELETED":
         return None
@@ -404,7 +427,6 @@ def volume_snapshot_create(
         payload["description"] = description
 
     print(f"\nCreating snapshot for volume '{volume_id}'...")
-    print(f"Request Payload: {json.dumps(payload, indent=2)}\n")
 
     path = f"/cloud/project/{client.project_id}/volume/{volume_id}/snapshot"
     snapshot_data = client.request("POST", path, payload)
@@ -491,6 +513,8 @@ def volume_snapshot_delete(
         snapshot_id: Snapshot ID
         client: Optional OVHClient instance (uses default if not provided)
     """
+    import ovh
+
     client = client or _get_default_client()
     print(f"\nDeleting volume snapshot {snapshot_id}...")
     path = f"/cloud/project/{client.project_id}/volume/snapshot/{snapshot_id}"
@@ -508,6 +532,142 @@ def volume_snapshot_delete(
         )
 
 
+def volume_backup_create(
+    volume_id: str,
+    name: str,
+    region: str,
+    wait: bool = True,
+    client: OVHClient | None = None,
+) -> VolumeBackup:
+    """
+    Create a backup of an OVHcloud volume.
+
+    Args:
+        volume_id: For which to create the backup
+        name: backup name
+        region: ovh region
+        client: Optional OVHClient instance (uses default if not provided)
+
+    Returns:
+        VolumeBackup object with details about the created backup
+    """
+    client = client or _get_default_client()
+
+    payload: dict[str, Any] = {
+        "volumeId": volume_id,
+        "name": name,
+    }
+
+    print(f"\nCreating backup for volume '{volume_id}'...")
+
+    path = f"/cloud/project/{client.project_id}/region/{region}/volumeBackup"
+    backup_data = client.request("POST", path, payload)
+    backup = VolumeBackup(**backup_data)
+
+    print(f"Backup created with ID: {backup.id}")
+
+    if wait:
+        _wait_for_item_ready(
+            client,
+            "volume backup",
+            volume_backup_get,
+            [backup.id, region],
+            "ok",
+        )
+
+    return backup
+
+
+def volume_backup_get(
+    backup_id: str,
+    region: str,
+    client: OVHClient | None = None,
+) -> VolumeBackup | None:
+    """
+    Get detailed information about a volume backup.
+
+    Args:
+        backup_id: Backup ID
+        client: Optional OVHClient instance (uses default if not provided)
+
+    Returns:
+        VolumeBackup object with details about the backup
+    """
+    client = client or _get_default_client()
+    path = (
+        f"/cloud/project/{client.project_id}/region/{region}/volumeBackup/{backup_id}"
+    )
+
+    response = client.request("GET", path)
+
+    if response.get("status") == "DELETED":
+        return None
+
+    return VolumeBackup(**response)
+
+
+def volume_backup_list(
+    region: str,
+    client: OVHClient | None = None,
+) -> list[VolumeBackup]:
+    """
+    List all volume backups in the project.
+
+    Args:
+        region: Optional region filter
+        client: Optional OVHClient instance (uses default if not provided)
+
+    Returns:
+        List of VolumeBackup objects
+    """
+    client = client or _get_default_client()
+    path = f"/cloud/project/{client.project_id}/region/{region}/volumeBackup"
+
+    # Prepare query parameters
+    params = {}
+    if region:
+        params["region"] = region
+
+    response = client.request("GET", path, params)
+    return [
+        VolumeBackup(**backup)
+        for backup in response
+        if backup.get("status") != "DELETED"
+    ]
+
+
+def volume_backup_delete(
+    backup_id: str,
+    region: str,
+    wait: bool = True,
+    client: OVHClient | None = None,
+) -> None:
+    """
+    Delete a volume backup.
+
+    Args:
+        backup_id: Backup ID
+        client: Optional OVHClient instance (uses default if not provided)
+    """
+    import ovh
+
+    client = client or _get_default_client()
+    print(f"\nDeleting volume backup {backup_id}...")
+    path = (
+        f"/cloud/project/{client.project_id}/region/{region}/volumeBackup/{backup_id}"
+    )
+    try:
+        client.request("DELETE", path)
+        print("Volume backup deletion initiated.")
+    except ovh.ResourceNotFoundError:
+        print("Could not find volume backup for deletion. Maybe it is already deleted?")
+
+    if wait:
+        _wait_for_item_deleted(
+            client, "volume backup", volume_backup_get, [backup_id, region]
+        )
+
+
 class InstanceFlavorCapability(BaseSchema):
     enabled: bool
     name: str
@@ -531,7 +691,7 @@ class InstanceFlavor(BaseSchema):
 
 
 class InstanceImage(BaseSchema):
-    creation_date: str
+    creation_date: datetime
     id: str
     min_disk: int
     min_ram: int
@@ -573,10 +733,10 @@ class Instance(BaseSchema):
     name: str
     status: str
     region: str
-    created: str
+    created: datetime
     ip_addresses: list[InstanceIpAddress]
     monthly_billing: InstanceMonthlyBilling | None
-    plan_code: str
+    plan_code: str | None
     operation_ids: list[str]
     current_month_outgoing_traffic: int | None = None
     availability_zone: str | None = None
@@ -671,7 +831,6 @@ def instance_create(
         ]
 
     print(f"\nCreating OVHcloud instance '{name}'...")
-    print(f"Request Payload: {json.dumps(payload, indent=2)}\n")
 
     path = f"/cloud/project/{client.project_id}/instance"
     instance_data = client.request("POST", path, payload)
@@ -730,20 +889,27 @@ def instance_start(
     client: OVHClient | None = None,
 ) -> None:
     """
-    Delete an instance.
+    Start an instance.
 
     Args:
         instance_id: Instance ID
-        wait: Wait for instance to be deleted
+        wait: Wait for instance to be started
         client: Optional OVHClient instance (uses default if not provided)
     """
+    instance = instance_get(instance_id)
+
+    if not instance:
+        raise ValueError(f"Could not get instance with id: {instance_id}")
+
+    if instance.status in ["ACTIVE"]:
+        print("Instance already started.")
+        return
+
     client = client or _get_default_client()
-    print(f"\nDeleting instance {instance_id}...")
+    print(f"\nStarting instance {instance_id}...")
     path = f"/cloud/project/{client.project_id}/instance/{instance_id}/start"
 
-    instance_data = client.request("POST", path)
-
-    instance = Instance(**instance_data)
+    client.request("POST", path)
 
     print("Instance start initiated.")
 
@@ -752,13 +918,55 @@ def instance_start(
             client,
             "instance",
             instance_get,
-            [instance.id],
+            [instance_id],
             "ACTIVE",
         )
 
     print("Instance started.")
 
 
+def instance_stop(
+    instance_id: str,
+    wait: bool = True,
+    client: OVHClient | None = None,
+) -> None:
+    """
+    Stop an instance.
+
+    Args:
+        instance_id: Instance ID
+        wait: Wait for instance to be stopped
+        client: Optional OVHClient instance (uses default if not provided)
+    """
+    instance = instance_get(instance_id)
+
+    if not instance:
+        raise ValueError(f"Could not get instance with id: {instance_id}")
+
+    if instance.status in ["SHUTOFF"]:
+        print("Instance already stopped.")
+        return
+
+    client = client or _get_default_client()
+    print(f"\nStopping instance {instance_id}...")
+    path = f"/cloud/project/{client.project_id}/instance/{instance_id}/stop"
+
+    client.request("POST", path)
+
+    print("Instance stop initiated.")
+
+    if wait:
+        _wait_for_item_ready(
+            client,
+            "instance",
+            instance_get,
+            [instance_id],
+            "SHUTOFF",
+        )
+
+    print("Instance stopped.")
+
+
 def instance_delete(
     instance_id: str,
     wait: bool = True,
@@ -804,8 +1012,8 @@ class KubeCluster(BaseSchema):
     version: str
     status: str  # READY, UPDATING, RESETTING, SUSPENDING, REOPENING, DELETING, SUSPENDED, ERROR, USER_ERROR
     url: str
-    created_at: str  # ISO 8601 datetime string
-    updated_at: str  # ISO 8601 datetime string
+    created_at: datetime  # ISO 8601 datetime string
+    updated_at: datetime  # ISO 8601 datetime string
     control_plane_is_up_to_date: bool
     is_up_to_date: bool
     nodes_url: str
@@ -865,7 +1073,6 @@ def kube_cluster_create(
         cluster_payload["privateNetworkId"] = private_network_id
 
     print("\nCreating OVHcloud Kubernetes cluster...")
-    print(f"Request Payload: {json.dumps(cluster_payload, indent=2)}\n")
 
     cluster_path = f"/cloud/project/{client.project_id}/kube"
     cluster_data = client.request("POST", cluster_path, cluster_payload)
@@ -1015,8 +1222,8 @@ class KubeNodePool(BaseSchema):
     up_to_date_nodes: int
     min_nodes: int
     max_nodes: int
-    created_at: str  # ISO 8601 datetime string
-    updated_at: str  # ISO 8601 datetime string
+    created_at: datetime  # ISO 8601 datetime string
+    updated_at: datetime  # ISO 8601 datetime string
     size_status: KubeNodePoolStatus
     autoscale: bool = False
     monthly_billed: bool = False
@@ -1038,10 +1245,10 @@ class KubeNodePoolNode(BaseSchema):
     )
     version: str
     is_up_to_date: bool
-    created_at: str
-    updated_at: str
-    deployed_at: str
-    deleted_at: str | None = None
+    created_at: datetime
+    updated_at: datetime
+    deployed_at: datetime
+    deleted_at: datetime | None = None
 
 
 def kube_node_pool_list(
@@ -1085,7 +1292,6 @@ def kube_node_pool_create(
     }
 
     print(f"\nCreating node pool '{name}'...")
-    print(f"Request Payload: {json.dumps(node_pool_payload, indent=2)}\n")
 
     path = f"/cloud/project/{client.project_id}/kube/{cluster_id}/nodepool"
     node_pool_data = client.request("POST", path, node_pool_payload)
@@ -1145,7 +1351,6 @@ def kube_node_pool_update(
         raise ValueError("No update parameters provided")
 
     print(f"\nUpdating node pool {node_pool_id}...")
-    print(f"Request Payload: {json.dumps(update_payload, indent=2)}\n")
 
     path = (
         f"/cloud/project/{client.project_id}/kube/{cluster_id}/nodepool/{node_pool_id}"
